@@ -36,9 +36,14 @@ int main(int argc, char *argv[])
     DApplication::loadDXcbPlugin();
 
     DApplication a(argc, argv);
+    Dtk::Core::DLogManager::registerConsoleAppender();
+
+    a.loadTranslator();
     a.setAttribute(Qt::AA_EnableHighDpiScaling, true);
     a.setTheme("light");
-    Dtk::Core::DLogManager::registerConsoleAppender();
+#ifdef Q_OS_UNIX
+    a.setOOMScoreAdj(500);
+#endif
 
     if (!a.setSingleInstance("deepin-tool-kit-examples")) {
         qDebug() << "another instance is running!!";
