@@ -61,6 +61,10 @@
 
 #ifdef Q_OS_LINUX
 #include "startupnotificationmonitor.h"
+
+#include <DDBusSender>
+
+#include <QGSettings>
 #endif
 
 #define DXCB_PLUGIN_KEY "dxcb"
@@ -150,16 +154,24 @@ bool DApplicationPrivate::setSingleInstanceBySemaphore(const QString &key)
     singleInstance = tryAcquireSystemSemaphore(&ss);
 
     if (singleInstance) {
-        QtConcurrent::run([] {
+        QtConcurrent::run([this] {
+            QPointer<DApplication> that = q_func();
+
             while (ss.acquire() && singleInstance)
             {
-                if (qApp->startingUp() || qApp->closingDown()) {
+                if (!that) {
+                    return;
+                }
+
+                if (that->startingUp() || that->closingDown()) {
                     break;
                 }
 
                 ss.release(1);
 
-                Q_EMIT qApp->newInstanceStarted();
+                if (that) {
+                    Q_EMIT that->newInstanceStarted();
+                }
             }
         });
 
@@ -224,6 +236,10 @@ bool DApplicationPrivate::loadDtkTranslator(QList<QLocale> localeFallback)
     DPathBuf currentDir(QDir::currentPath());
     translateDirs << currentDir.join("translations");
 
+#ifdef DTK_STATIC_TRANSLATION
+    translateDirs << DPathBuf(":/dtk/translations");
+#endif
+
     return loadTranslator(translateDirs, dtkwidgetName, localeFallback);
 }
 
@@ -297,7 +313,15 @@ DApplication::DApplication(int &argc, char **argv) :
 {
     qputenv("QT_QPA_PLATFORM", QByteArray());
 
-    QPixmapCache::setCacheLimit(0);
+#ifdef Q_OS_LINUX
+    // set qpixmap cache limit
+    if (QGSettings::isSchemaInstalled("com.deepin.dde.dapplication"))
+    {
+        QGSettings gsettings("com.deepin.dde.dapplication", "/com/deepin/dde/dapplication/");
+        if (gsettings.keys().contains("qpixmapCacheLimit"))
+            QPixmapCache::setCacheLimit(gsettings.get("qpixmap-cache-limit").toInt());
+    }
+#endif
 }
 
 /**
@@ -325,6 +349,27 @@ void DApplication::setTheme(const QString &theme)
     d->setTheme(theme);
 }
 
+#ifdef Q_OS_UNIX
+/**
+ * @brief DApplication::setOOMScoreAdj set Out-Of-Memory score
+ * @param score vaild range is [-1000, 1000]
+ */
+void DApplication::setOOMScoreAdj(const int score)
+{
+    if (score > 1000 || score < -1000)
+        qWarning() << "OOM score adjustment value out of range: " << score;
+
+    QFile f("/proc/self/oom_score_adj");
+    if (!f.open(QIODevice::WriteOnly))
+    {
+        qWarning() << "OOM score adjust failed, open file error: " << f.errorString();
+        return;
+    }
+
+    f.write(std::to_string(score).c_str());
+}
+#endif
+
 /**
  * @brief DApplication::setSingleInstance marks this application to be single instanced.
  * @param key is used as the unique ID of every application.
@@ -389,6 +434,10 @@ bool DApplication::loadTranslator(QList<QLocale> localeFallback)
     DPathBuf currentDir(QDir::currentPath());
     translateDirs << currentDir.join("translations");
 
+#ifdef DTK_STATIC_TRANSLATION
+    translateDirs << DPathBuf(":/dtk/translations");
+#endif
+
     return d->loadTranslator(translateDirs, appName, localeFallback);
 }
 
@@ -408,7 +457,12 @@ bool DApplication::loadDXcbPlugin()
 
 bool DApplication::isDXcbPlatform()
 {
-    return qApp && (qApp->platformName() == DXCB_PLUGIN_KEY || qApp->property(DXCB_PLUGIN_SYMBOLIC_PROPERTY).toBool());
+    if (!qApp)
+        return false;
+
+    static bool _is_dxcb = qApp->platformName() == DXCB_PLUGIN_KEY || qApp->property(DXCB_PLUGIN_SYMBOLIC_PROPERTY).toBool();
+
+    return _is_dxcb;
 }
 
 int DApplication::buildDtkVersion()
@@ -421,6 +475,26 @@ int DApplication::runtimeDtkVersion()
     return DTK_VERSION;
 }
 
+void DApplication::registerDDESession()
+{
+#ifdef Q_OS_LINUX
+    QString envName("DDE_SESSION_PROCESS_COOKIE_ID");
+
+    QByteArray cookie = qgetenv(envName.toUtf8().data());
+    qunsetenv(envName.toUtf8().data());
+
+    if (!cookie.isEmpty()) {
+        DDBusSender()
+                .service("com.deepin.SessionManager")
+                .path("/com/deepin/SessionManager")
+                .interface("com.deepin.SessionManager")
+                .method("Register")
+                .arg(QString(cookie))
+                .call();
+    }
+#endif
+}
+
 /**
  * @brief DApplication::productName returns the product name of this application.
  *
@@ -572,6 +646,18 @@ void DApplication::setApplicationAcknowledgementPage(const QString &link)
     d->acknowledgementPage = link;
 }
 
+bool DApplication::applicationAcknowledgementVisible() const
+{
+    D_DC(DApplication);
+    return d->acknowledgementPageVisible;
+}
+
+void DApplication::setApplicationAcknowledgementVisible(bool visible)
+{
+    D_D(DApplication);
+    d->acknowledgementPageVisible = visible;
+}
+
 /**
  * @brief DApplication::aboutDialog returns the about dialog of this application.
  *
@@ -664,27 +750,14 @@ void DApplication::handleHelpAction()
     QDBusInterface manual("com.deepin.Manual.Open",
                           "/com/deepin/Manual/Open",
                           "com.deepin.Manual.Open");
-    if (manual.isValid()) {
-        manual.asyncCall("ShowManual", appid);
+    QDBusReply<void> reply = manual.call("ShowManual", appid);
+    if (reply.isValid())  {
+        qDebug() << "call com.deepin.Manual.Open success";
         return;
     }
-
+    qDebug() << "call com.deepin.Manual.Open failed" << reply.error();
     // fallback to old interface
-    if (!qgetenv("FLATPAK_APPID").isEmpty()) {
-        appid = qgetenv("FLATPAK_APPID");
-        QDBusInterface legacydman("com.deepin.dman",
-                                  "/com/deepin/dman",
-                                  "com.deepin.dman");
-        if (legacydman.isValid()) {
-            // will block application
-            legacydman.call("ShowManual", appid);
-            return;
-        }
-
-        qWarning() << "can not call dman dbus interface";
-    } else {
-        QProcess::startDetached("dman", QStringList() << appid);
-    }
+    QProcess::startDetached("dman", QStringList() << appid);
 #else
     qWarning() << "not support dman now";
 #endif
@@ -719,6 +792,7 @@ void DApplication::handleAboutAction()
     if (!applicationAcknowledgementPage().isEmpty()) {
         aboutDialog->setAcknowledgementLink(applicationAcknowledgementPage());
     }
+    aboutDialog->setAcknowledgementVisible(d->acknowledgementPageVisible);
 
     aboutDialog->exec();
     aboutDialog->deleteLater();
