@@ -30,6 +30,8 @@
 #include <QStyleFactory>
 #include <QWheelEvent>
 #include <QtMath>
+#include <QPointer>
+
 #include "dhidpihelper.h"
 
 DCORE_USE_NAMESPACE
@@ -48,10 +50,10 @@ public:
     int getTopRenderOffset();
     void sortItemsByColumn(int column, bool descendingSort);
 
-    DSimpleListItem *lastHoverItem;
-    DSimpleListItem *lastSelectItem;
-    DSimpleListItem *drawHoverItem;
-    DSimpleListItem *mouseHoverItem;
+    QPointer<DSimpleListItem> lastHoverItem;
+    QPointer<DSimpleListItem> lastSelectItem;
+    QPointer<DSimpleListItem> drawHoverItem;
+    QPointer<DSimpleListItem> mouseHoverItem;
     QList<DSimpleListItem*> *listItems;
     QList<DSimpleListItem*> *renderItems;
     QList<DSimpleListItem*> *selectionItems;
@@ -66,6 +68,8 @@ public:
     bool mouseAtScrollArea;
     bool mouseDragScrollbar;
     bool drawFrame;
+    bool isKeepSelectWhenClickBlank;
+    bool isSingleSelect;
     int alwaysVisibleColumn;
     int clipRadius;
     int defaultSortingColumn;
@@ -144,6 +148,8 @@ DSimpleListView::DSimpleListView(QWidget *parent) : QWidget(parent), DObject(*ne
     d->mouseAtScrollArea = false;
     d->mouseDragScrollbar = false;
     d->drawFrame = false;
+    d->isKeepSelectWhenClickBlank = false;
+    d->isSingleSelect = false;
     d->scrollbarDefaultWidth = 4;
     d->scrollbarDragWidth = 8;
     d->scrollbarMinHeight = 30;
@@ -163,10 +169,10 @@ DSimpleListView::~DSimpleListView()
 {
     D_D(DSimpleListView);
 
-    delete d->lastHoverItem;
-    delete d->lastSelectItem;
-    delete d->drawHoverItem;
-    delete d->mouseHoverItem;
+    delete d->lastHoverItem.data();
+    delete d->lastSelectItem.data();
+    delete d->drawHoverItem.data();
+    delete d->mouseHoverItem.data();
     delete d->listItems;
     delete d->renderItems;
     delete d->selectionItems;
@@ -332,6 +338,13 @@ void DSimpleListView::clearSelections(bool clearLastSelection)
     }
 }
 
+QList<DSimpleListItem*> DSimpleListView::getSelections()
+{
+    D_D(DSimpleListView);
+
+    return *d->selectionItems;
+}
+
 void DSimpleListView::refreshItems(QList<DSimpleListItem*> items)
 {
     D_D(DSimpleListView);
@@ -414,22 +427,39 @@ void DSimpleListView::search(QString content)
     repaint();
 }
 
+void DSimpleListView::setSingleSelect(bool singleSelect)
+{
+    D_D(DSimpleListView);
+        
+    d->isSingleSelect = singleSelect;
+}
+
+void DSimpleListView::keepSelectWhenClickBlank(bool keep)
+{
+    D_D(DSimpleListView);
+    
+    d->isKeepSelectWhenClickBlank = keep;
+}
+
 void DSimpleListView::selectAllItems()
 {
     D_D(DSimpleListView);
+    
+    if (!d->isSingleSelect) {
+        // Record old render offset to control scrollbar whether display.
+        d->oldRenderOffset = d->renderOffset;
 
-    // Record old render offset to control scrollbar whether display.
-    d->oldRenderOffset = d->renderOffset;
+        // Select all items.
+        clearSelections();
+        addSelections(*d->renderItems);
 
-    // Select all items.
-    clearSelections();
-    addSelections(*d->renderItems);
+        // Scroll to top.
+        d->renderOffset = d->getTopRenderOffset();
 
-    // Scroll to top.
-    d->renderOffset = d->getTopRenderOffset();
+        // Repaint.
+        repaint();
+    }
 
-    // Repaint.
-    repaint();
 }
 
 void DSimpleListView::selectFirstItem()
@@ -491,73 +521,87 @@ void DSimpleListView::selectNextItem()
 void DSimpleListView::shiftSelectPageDown()
 {
     D_D(DSimpleListView);
-
-    shiftSelectNextItemWithOffset(getScrollAreaHeight() / d->rowHeight);
+    
+    if (!d->isSingleSelect) {
+        shiftSelectNextItemWithOffset(getScrollAreaHeight() / d->rowHeight);
+    }
 }
 
 void DSimpleListView::shiftSelectPageUp()
 {
     D_D(DSimpleListView);
-
-    shiftSelectPrevItemWithOffset(getScrollAreaHeight() / d->rowHeight);
+    
+    if (!d->isSingleSelect) {
+        shiftSelectPrevItemWithOffset(getScrollAreaHeight() / d->rowHeight);
+    }
 }
 
 void DSimpleListView::shiftSelectToEnd()
 {
     D_D(DSimpleListView);
+    
+    if (!d->isSingleSelect) {
+        // Select last item if nothing selected yet.
+        if (d->selectionItems->empty()) {
+            selectLastItem();
+        }
+        // Select items from last selected item to last item.
+        else {
+            // Found last selected index and do select operation.
+            int lastSelectionIndex = d->renderItems->indexOf(d->lastSelectItem);
+            shiftSelectItemsWithBound(lastSelectionIndex, d->renderItems->count() - 1);
 
-    // Select last item if nothing selected yet.
-    if (d->selectionItems->empty()) {
-        selectLastItem();
+            // Scroll to bottom.
+            d->renderOffset = getBottomRenderOffset();
+
+            // Repaint.
+            repaint();
+        }
     }
-    // Select items from last selected item to last item.
-    else {
-        // Found last selected index and do select operation.
-        int lastSelectionIndex = d->renderItems->indexOf(d->lastSelectItem);
-        shiftSelectItemsWithBound(lastSelectionIndex, d->renderItems->count() - 1);
 
-        // Scroll to bottom.
-        d->renderOffset = getBottomRenderOffset();
-
-        // Repaint.
-        repaint();
-    }
 }
 
 void DSimpleListView::shiftSelectToHome()
 {
     D_D(DSimpleListView);
+    
+    if (!d->isSingleSelect) {
+        // Select first item if nothing selected yet.
+        if (d->selectionItems->empty()) {
+            selectFirstItem();
+        }
+        // Select items from last selected item to first item.
+        else {
+            // Found last selected index and do select operation.
+            int lastSelectionIndex = d->renderItems->indexOf(d->lastSelectItem);
+            shiftSelectItemsWithBound(0, lastSelectionIndex);
 
-    // Select first item if nothing selected yet.
-    if (d->selectionItems->empty()) {
-        selectFirstItem();
+            // Scroll to top.
+            d->renderOffset = d->getTopRenderOffset();
+
+            // Repaint.
+            repaint();
+        }
     }
-    // Select items from last selected item to first item.
-    else {
-        // Found last selected index and do select operation.
-        int lastSelectionIndex = d->renderItems->indexOf(d->lastSelectItem);
-        shiftSelectItemsWithBound(0, lastSelectionIndex);
 
-        // Scroll to top.
-        d->renderOffset = d->getTopRenderOffset();
-
-        // Repaint.
-        repaint();
-    }
 }
 
 void DSimpleListView::shiftSelectToNext()
 {
     D_D(DSimpleListView);
-
-    shiftSelectNextItemWithOffset(1);
+    
+    if (!d->isSingleSelect) {
+        shiftSelectNextItemWithOffset(1);
+    }
 }
 
 void DSimpleListView::shiftSelectToPrev()
 {
     D_D(DSimpleListView);
-
-    shiftSelectPrevItemWithOffset(1);
+    
+    if (!d->isSingleSelect) {
+        shiftSelectPrevItemWithOffset(1);
+    }
 }
 
 void DSimpleListView::scrollPageDown()
@@ -612,6 +656,12 @@ void DSimpleListView::ctrlScrollToEnd()
 
 void DSimpleListView::leaveEvent(QEvent * event)
 {
+    D_D(DSimpleListView);
+
+    d->lastHoverItem.clear();
+    d->drawHoverItem.clear();
+    d->mouseHoverItem.clear();
+
     hideScrollbar();
 
     QWidget::leaveEvent(event);
@@ -883,14 +933,16 @@ void DSimpleListView::mousePressEvent(QMouseEvent *mouseEvent)
         int pressItemIndex = (d->renderOffset + mouseEvent->y() - d->titleHeight) / d->rowHeight;
 
         if (pressItemIndex >= d->renderItems->count()) {
-            clearSelections();
+            if (!d->isKeepSelectWhenClickBlank) {
+                clearSelections();
+            }
 
             repaint();
         } else {
             if (mouseEvent->button() == Qt::LeftButton) {
                 if (pressItemIndex < d->renderItems->count()) {
                     // Scattered selection of items when press ctrl modifier.
-                    if (mouseEvent->modifiers() == Qt::ControlModifier) {
+                    if (!d->isSingleSelect && mouseEvent->modifiers() == Qt::ControlModifier) {
                         DSimpleListItem *item = (*d->renderItems)[pressItemIndex];
 
                         if (d->selectionItems->contains(item)) {
@@ -902,7 +954,7 @@ void DSimpleListView::mousePressEvent(QMouseEvent *mouseEvent)
                         }
                     }
                     // Continuous selection of items when press shift modifier.
-                    else if ((mouseEvent->modifiers() == Qt::ShiftModifier) && !d->selectionItems->empty()) {
+                    else if (!d->isSingleSelect && (mouseEvent->modifiers() == Qt::ShiftModifier) && !d->selectionItems->empty()) {
                         int lastSelectionIndex = d->renderItems->indexOf(d->lastSelectItem);
                         int selectionStartIndex = std::min(pressItemIndex, lastSelectionIndex);
                         int selectionEndIndex = std::max(pressItemIndex, lastSelectionIndex);
