@@ -21,8 +21,12 @@
 #ifndef DSTYLEOPTION_H
 #define DSTYLEOPTION_H
 
+#include <dtkwidget_global.h>
+
 #include <QStyleOption>
 
+DWIDGET_BEGIN_NAMESPACE
+
 class DStyleOption
 {
 public:
@@ -39,4 +43,21 @@ public:
     void init(QWidget *widget) override;
 };
 
+class DStyleOptionLineEdit : public DStyleOption
+{
+public:
+    enum LineEditFeature {
+        Alert = 0x1,
+        IconButton = 0x2
+    };
+    Q_DECLARE_FLAGS(LineEditFeatures, LineEditFeature)
+
+    void init(QWidget *widget) override;
+
+    LineEditFeatures features = 0;
+    QRect iconButtonRect;
+};
+
+DWIDGET_END_NAMESPACE
+
 #endif // DSTYLEOPTION_H
