@@ -24,7 +24,6 @@
 #include "dthememanager.h"
 
 #include <QPainter>
-#include <QApplication>
 #include <QMouseEvent>
 #include <QStyleOption>
 #include <QStylePainter>
@@ -32,9 +31,19 @@
 #include <QDrag>
 #include <QMimeData>
 #include <QDragMoveEvent>
+#include <QTimer>
 #include <QDebug>
 
 #include <private/qtabbar_p.h>
+#define private public
+#define protected public
+#include <private/qdnd_p.h>
+#include <private/qsimpledrag_p.h>
+#include <private/qshapedpixmapdndwindow_p.h>
+#undef private
+#undef protected
+
+#include "dapplication.h"
 
 DWIDGET_BEGIN_NAMESPACE
 
@@ -122,6 +131,13 @@ public:
         connect(this, &QTabBar::tabMoved, this, [this] (int from, int to) {
             tabMinimumSize.move(from, to);
             tabMaximumSize.move(from, to);
+
+            if (dd()->validIndex(ghostTabIndex)) {
+                if (from == ghostTabIndex)
+                    ghostTabIndex = to;
+                else if (to == ghostTabIndex)
+                    ghostTabIndex = from;
+            }
         });
 
         setAcceptDrops(true);
@@ -215,11 +231,9 @@ public:
     void dragMoveEvent(QDragMoveEvent *e) override;
     void dropEvent(QDropEvent *e) override;
     void showEvent(QShowEvent *e) override;
-    void resizeEvent(QResizeEvent *e) override;
 
     QSize tabSizeHint(int index) const override;
     QSize minimumTabSizeHint(int index) const override;
-    virtual QSize maximumTabSizeHint(int index) const;
 
     void tabInserted(int index) override;
     void tabRemoved(int index) override;
@@ -227,6 +241,8 @@ public:
 
     void initStyleOption(QStyleOptionTab *option, int tabIndex) const;
 
+     QTabBarPrivate *dd() const;
+
     Q_SLOT void startDrag(int tabIndex);
 
     void setupMovableTab();
@@ -241,11 +257,13 @@ public:
     void stopAutoScrollTabs();
     void ensureScrollTabsAnimation();
 
-    void updateTabSize();
     void startTabFlash();
 
     void setDragingFromOther(bool v);
-    int tabInsertIndexFromMouse(const QPoint &pos);
+    int tabInsertIndexFromMouse(QPoint pos);
+
+    void startMove(int index);
+    void stopMove();
 
     QList<QSize> tabMinimumSize;
     QList<QSize> tabMaximumSize;
@@ -258,6 +276,8 @@ public:
     bool dragingFromOther = false;
     // 记录当前drag过来的对象是否可以当做新标签页插入
     bool canInsertFromDrag = false;
+    // 为true忽略drag move事件
+    bool ignoreDragEvent = false;
 
     QColor maskColor;
     QColor flashColor;
@@ -286,12 +306,16 @@ public:
     // 备份启动tab move时的QTabBarPrivate中的这两个值
     int scrollOffset;
     QPoint dragStartPosition;
+
+    int ghostTabIndex = -1;
 };
 
 void DTabBarPrivate::startDrag(int tabIndex)
 {
     Qt::DropAction action = drag->exec(Qt::MoveAction | Qt::CopyAction, Qt::CopyAction);
 
+    Q_EMIT q_func()->dragEnd(action);
+
     if (action == Qt::IgnoreAction) {
         Q_EMIT q_func()->tabReleaseRequested(tabIndex);
     } else if (drag->target() != this) {
@@ -433,7 +457,13 @@ void DTabBarPrivate::setupDragableTab()
     drag->setMimeData(mime_data);
     drag->setHotSpot(hotspot);
 
+    qRegisterMetaType<Qt::DropAction>();
+
     QMetaObject::invokeMethod(this, "startDrag", Qt::QueuedConnection, Q_ARG(int, d->pressedIndex));
+    QMetaObject::invokeMethod(q, "dragStarted", Qt::QueuedConnection);
+    QMetaObject::invokeMethod(q, "dragActionChanged", Qt::QueuedConnection, Q_ARG(Qt::DropAction, Qt::IgnoreAction));
+
+    connect(drag, &QDrag::actionChanged, q, &DTabBar::dragActionChanged);
 }
 
 void DTabBarPrivate::slide(int from, int to)
@@ -705,28 +735,6 @@ void DTabBarPrivate::ensureScrollTabsAnimation()
     });
 }
 
-void DTabBarPrivate::updateTabSize()
-{
-    QTabBarPrivate *d = reinterpret_cast<QTabBarPrivate *>(qGetPtrHelper(d_ptr));
-
-    bool is_vertical = verticalTabs(d->shape);
-
-    for (int i = 0; i < d->tabList.count(); ++i) {
-        QTabBarPrivate::Tab &tab = d->tabList[i];
-        const QSize &size_hint = tabSizeHint(i);
-        const QSize old_size = tab.rect.size();
-
-        if (is_vertical) {
-            tab.rect.setWidth(qMax(size_hint.width(), width()));
-        } else {
-            tab.rect.setHeight(qMax(size_hint.height(), height()));
-        }
-
-        if (tab.rect.size() != old_size)
-            layoutTab(i);
-    }
-}
-
 void DTabBarPrivate::startTabFlash()
 {
     QVariantAnimation *animation = new QVariantAnimation(this);
@@ -785,8 +793,14 @@ void DTabBarPrivate::setDragingFromOther(bool v)
     topFullWidget->raise();
 }
 
-int DTabBarPrivate::tabInsertIndexFromMouse(const QPoint &pos)
+int DTabBarPrivate::tabInsertIndexFromMouse(QPoint pos)
 {
+    if (pos.y() == height())
+        pos.setY(pos.y() - 1);
+
+    if (pos.x() == width())
+        pos.setX(pos.x() - 1);
+
     int current = tabAt(pos);
 
     QTabBarPrivate *d = reinterpret_cast<QTabBarPrivate *>(qGetPtrHelper(d_ptr));
@@ -794,9 +808,9 @@ int DTabBarPrivate::tabInsertIndexFromMouse(const QPoint &pos)
 
     if (!d->validIndex(current)){
         if (vertical)
-            current = pos.y() >= height() ? -1 : 0;
+            current = pos.y() >= height() ? count() - 1 : 0;
         else
-            current = pos.x() >= width() ? -1 : 0;
+            current = pos.x() >= width() ? count() - 1 : 0;
     }
 
     const QRect &current_rect = tabRect(current);
@@ -809,6 +823,50 @@ int DTabBarPrivate::tabInsertIndexFromMouse(const QPoint &pos)
     }
 }
 
+void DTabBarPrivate::startMove(int index)
+{
+    if (dd()->dragInProgress)
+        return;
+
+    dd()->pressedIndex = index;
+    makeVisible(index);
+
+    const QRect &index_rect = tabRect(index);
+
+    dd()->dragStartPosition = index_rect.center();
+
+    QVariantAnimation *mouse_animation = new QVariantAnimation(this);
+
+    mouse_animation->setDuration(100);
+    mouse_animation->setEasingCurve(QEasingCurve::OutSine);
+    mouse_animation->setStartValue(QCursor::pos());
+    mouse_animation->setEndValue(mapToGlobal(index_rect.center()));
+
+    connect(mouse_animation, &QVariantAnimation::valueChanged, this, [] (const QVariant &value) {
+        const QPoint pos = value.toPoint();
+
+        QCursor::setPos(pos.x(), pos.y());
+    });
+
+    connect(mouse_animation, &QVariantAnimation::finished, this, [this, mouse_animation] {
+        mouse_animation->deleteLater();
+        ignoreDragEvent = false;
+    });
+
+    ignoreDragEvent = true;
+    mouse_animation->start();
+}
+
+void DTabBarPrivate::stopMove()
+{
+    QMouseEvent event(QEvent::MouseButtonRelease, mapFromGlobal(QCursor::pos()),
+                      Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
+    bool movable = isMovable();
+    setMovable(true);
+    mouseReleaseEvent(&event);
+    setMovable(movable);
+}
+
 bool DTabBarPrivate::eventFilter(QObject *watched, QEvent *event)
 {
     QTabBarPrivate *d = reinterpret_cast<QTabBarPrivate *>(qGetPtrHelper(d_ptr));
@@ -834,6 +892,10 @@ bool DTabBarPrivate::eventFilter(QObject *watched, QEvent *event)
         switch (event->type()) {
         case QEvent::Show:
             rightScrollButton->show();
+            // ###(zccrs): 当主窗口从最大化还原后，此button的大小会被布局改为 0x40 （原因不明）
+            QTimer::singleShot(100, rightScrollButton, [this] {
+                rightScrollButton->resize(rightScrollButton->sizeHint().width(), rightScrollButton->height());
+            });
             break;
         case QEvent::Hide:
             rightScrollButton->hide();
@@ -970,7 +1032,7 @@ void DTabBarPrivate::mouseMoveEvent(QMouseEvent *event)
 {
     QTabBarPrivate *d = reinterpret_cast<QTabBarPrivate *>(qGetPtrHelper(d_ptr));
 
-    if (!d->movable)
+    if (!d->movable && !d->validIndex(ghostTabIndex))
         return;
 
     // Be safe!
@@ -992,7 +1054,7 @@ void DTabBarPrivate::mouseMoveEvent(QMouseEvent *event)
     }
 
     if (!drag && valid_pressed_index) {
-        if (offset_y > startDragDistance) {
+        if (offset_y > startDragDistance && !d->validIndex(ghostTabIndex)) {
             setupDragableTab();
         }
     }
@@ -1017,7 +1079,7 @@ void DTabBarPrivate::mouseMoveEvent(QMouseEvent *event)
         scrollOffset = d->scrollOffset;
         dragStartPosition = d->dragStartPosition;
 
-        // Auto scroll tags
+        // Auto scroll tabs
         autoScrollTabs(event->pos());
     }
 }
@@ -1116,27 +1178,27 @@ void DTabBarPrivate::showEvent(QShowEvent *e)
     QTabBar::showEvent(e);
 }
 
-void DTabBarPrivate::resizeEvent(QResizeEvent *e)
-{
-    updateTabSize();
-
-    QTabBar::resizeEvent(e);
-}
-
 QSize DTabBarPrivate::tabSizeHint(int index) const
 {
     D_QC(DTabBar);
 
+    if (qApp->buildDtkVersion() > DTK_VERSION_CHECK(2, 0, 8, 1))
+        return q->tabSizeHint(index);
+
+    return q->DTabBar::tabSizeHint(index);
+}
+
+QSize DTabBarPrivate::minimumTabSizeHint(int index) const
+{
+    D_QC(DTabBar);
+
+    const QSize &min = qApp->buildDtkVersion() > DTK_VERSION_CHECK(2, 0, 8, 1) ? q->minimumTabSizeHint(index) : q->DTabBar::minimumTabSizeHint(index);
+
+    if (min.isValid())
+        return min;
+
     QSize size = QTabBar::tabSizeHint(index);
-
-    if (index >= tabMaximumSize.count())
-        return size;
-
-    const QSize &min = tabMinimumSize.at(index);
-    const QSize &max = tabMaximumSize.at(index);
-
-    size.setWidth(qMax(size.width(), min.width()));
-    size.setHeight(qMax(size.height(), min.height()));
+    const QSize &max = qApp->buildDtkVersion() > DTK_VERSION_CHECK(2, 0, 8, 1) ? q->maximumTabSizeHint(index) : q->DTabBar::maximumTabSizeHint(index);
 
     if (max.isValid()) {
         size.setWidth(qMin(size.width(), max.width()));
@@ -1146,47 +1208,34 @@ QSize DTabBarPrivate::tabSizeHint(int index) const
     return size;
 }
 
-QSize DTabBarPrivate::minimumTabSizeHint(int index) const
-{
-    const QSize &min = tabMinimumSize.value(index);
-
-    if (min.isValid())
-        return min;
-
-    return QTabBar::minimumTabSizeHint(index);
-}
-
-QSize DTabBarPrivate::maximumTabSizeHint(int index) const
-{
-    const QSize &max = tabMaximumSize.value(index);
-
-    if (max.isValid())
-        return max;
-
-    return QTabBar::tabSizeHint(index);
-}
-
 void DTabBarPrivate::tabInserted(int index)
 {
-    tabMaximumSize.insert(index, QSize());
-    tabMinimumSize.insert(index, QSize());
+    D_Q(DTabBar);
 
-    QTabBar::tabInserted(index);
+    if (qApp->buildDtkVersion() > DTK_VERSION_CHECK(2, 0, 8, 1))
+        q->tabInserted(index);
+    else
+        q->DTabBar::tabInserted(index);
 }
 
 void DTabBarPrivate::tabRemoved(int index)
 {
-    tabMaximumSize.removeAt(index);
-    tabMinimumSize.removeAt(index);
+    D_Q(DTabBar);
 
-    QTabBar::tabInserted(index);
+    if (qApp->buildDtkVersion() > DTK_VERSION_CHECK(2, 0, 8, 1))
+        q->tabRemoved(index);
+    else
+        q->DTabBar::tabRemoved(index);
 }
 
 void DTabBarPrivate::tabLayoutChange()
 {
-    updateTabSize();
+    D_Q(DTabBar);
 
-    QTabBar::tabLayoutChange();
+    if (qApp->buildDtkVersion() > DTK_VERSION_CHECK(2, 0, 8, 1))
+        q->tabLayoutChange();
+    else
+        q->DTabBar::tabLayoutChange();
 }
 
 void DTabBarPrivate::initStyleOption(QStyleOptionTab *option, int tabIndex) const
@@ -1194,6 +1243,11 @@ void DTabBarPrivate::initStyleOption(QStyleOptionTab *option, int tabIndex) cons
     QTabBar::initStyleOption(option, tabIndex);
 }
 
+QTabBarPrivate *DTabBarPrivate::dd() const
+{
+    return reinterpret_cast<QTabBarPrivate *>(qGetPtrHelper(d_ptr));
+}
+
 DTabBar::DTabBar(QWidget *parent)
     : QWidget(parent)
     , DObject(*new DTabBarPrivate(this))
@@ -1205,14 +1259,28 @@ void DTabBar::setTabMinimumSize(int index, const QSize &size)
 {
     D_D(DTabBar);
 
+    if (d->tabMinimumSize.at(index) == size)
+        return;
+
     d->tabMinimumSize[index] = size;
+
+    QTabBarPrivate *dd = reinterpret_cast<QTabBarPrivate *>(qGetPtrHelper(d->d_ptr));
+
+    dd->layoutDirty = true;
 }
 
 void DTabBar::setTabMaximumSize(int index, const QSize &size)
 {
     D_D(DTabBar);
 
+    if (d->tabMaximumSize.at(index) == size)
+        return;
+
     d->tabMaximumSize[index] = size;
+
+    QTabBarPrivate *dd = reinterpret_cast<QTabBarPrivate *>(qGetPtrHelper(d->d_ptr));
+
+    dd->layoutDirty = true;
 }
 
 bool DTabBar::visibleAddButton() const
@@ -1531,6 +1599,16 @@ QColor DTabBar::flashColor() const
     return d_func()->flashColor;
 }
 
+QWindow *DTabBar::dragIconWindow() const
+{
+    for (QWindow *w : qGuiApp->allWindows()) {
+        if (w->inherits("QShapedPixmapWindow"))
+            return w;
+    }
+
+    return nullptr;
+}
+
 void DTabBar::setCurrentIndex(int index)
 {
     d_func()->setCurrentIndex(index);
@@ -1559,6 +1637,25 @@ void DTabBar::setFlashColor(QColor flashColor)
     d_func()->flashColor = flashColor;
 }
 
+void DTabBar::startDrag(int index)
+{
+    D_D(DTabBar);
+
+    d->dd()->pressedIndex = index;
+    d->setupDragableTab();
+}
+
+void DTabBar::stopDrag(Qt::DropAction action)
+{
+    if (QBasicDrag *drag = dynamic_cast<QBasicDrag*>(QDragManager::self()->m_platformDrag)) {
+        drag->cancel();
+        drag->m_executed_drop_action = action;
+
+        if (drag->m_eventLoop)
+            drag->m_eventLoop->quit();
+    }
+}
+
 void DTabBar::dragEnterEvent(QDragEnterEvent *e)
 {
     D_D(DTabBar);
@@ -1571,6 +1668,16 @@ void DTabBar::dragEnterEvent(QDragEnterEvent *e)
     if (canInsertFromMimeData(index, e->mimeData())) {
         d->setDragingFromOther(true);
         e->acceptProposedAction();
+
+        // 插入一个虚拟的标签
+        if (e->source() != d) {
+            d->ghostTabIndex = index;
+            insertFromMimeDataOnDragEnter(index, e->mimeData());
+            // 延时启动startMove， 此时tabbar的大小还没有更新
+            QTimer::singleShot(10, [d, index] {
+                d->startMove(index);
+            });
+        }
     }
 }
 
@@ -1581,23 +1688,51 @@ void DTabBar::dragLeaveEvent(QDragLeaveEvent *e)
 
     d->setDragingFromOther(false);
     d->stopAutoScrollTabs();
+
+    if (d->dd()->validIndex(d->ghostTabIndex)) {
+        d->stopMove();
+        d->removeTab(d->ghostTabIndex);
+        d->ghostTabIndex = -1;
+    }
 }
 
 void DTabBar::dragMoveEvent(QDragMoveEvent *e)
 {
     D_D(DTabBar);
 
+    if (d->ignoreDragEvent)
+        return;
+
     if (e->source() == d)
         return QWidget::dragMoveEvent(e);
 
-    if (e->source() != d)
-        d->autoScrollTabs(d->mapFromParent(e->pos()));
-
-    int index = d->tabInsertIndexFromMouse(d->mapFromParent(e->pos()));
+    int index = d->dd()->validIndex(d->ghostTabIndex) ? d->ghostTabIndex : d->tabInsertIndexFromMouse(d->mapFromParent(e->pos()));
+    bool canInsert = false;
 
     if (canInsertFromMimeData(index, e->mimeData())) {
         d->setDragingFromOther(true);
         e->acceptProposedAction();
+        canInsert = true;
+    } else if (d->dd()->validIndex(d->ghostTabIndex)) {
+        d->stopMove();
+        d->removeTab(d->ghostTabIndex);
+        d->ghostTabIndex = -1;
+    }
+
+    if (e->source() != d) {
+        if (canInsert) {
+            if (d->dd()->validIndex(d->ghostTabIndex)) {
+                QMouseEvent event(QEvent::MouseMove, d->mapFromParent(e->pos()),
+                                  Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
+                d->mouseMoveEvent(&event);
+            } else {
+                d->ghostTabIndex = index;
+                insertFromMimeDataOnDragEnter(index, e->mimeData());
+                d->startMove(index);
+            }
+        } else {
+            d->autoScrollTabs(d->mapFromParent(e->pos()));
+        }
     }
 }
 
@@ -1616,7 +1751,21 @@ void DTabBar::dropEvent(QDropEvent *e)
     if (canInsertFromMimeData(index, e->mimeData())) {
         e->acceptProposedAction();
         e->setDropAction(Qt::MoveAction);
-        insertFromMimeData(index, e->mimeData());
+
+        if (d->dd()->validIndex(d->ghostTabIndex)) {
+            d->stopMove();
+            {
+                QSignalBlocker blocker(this);
+                Q_UNUSED(blocker)
+                d->removeTab(d->ghostTabIndex);
+            }
+            Q_EMIT tabIsRemoved(d->ghostTabIndex);
+
+            insertFromMimeData(d->ghostTabIndex, e->mimeData());
+            d->ghostTabIndex = -1;
+        } else {
+            insertFromMimeData(index, e->mimeData());
+        }
     }
 }
 
@@ -1632,6 +1781,13 @@ void DTabBar::resizeEvent(QResizeEvent *e)
     return QWidget::resizeEvent(e);
 }
 
+void DTabBar::startTabFlash(int index)
+{
+    d_func()->flashTabIndex = index;
+    d_func()->makeVisible(d_func()->flashTabIndex);
+    d_func()->startTabFlash();
+}
+
 void DTabBar::paintTab(QPainter *painter, int index, const QStyleOptionTab &option) const
 {
     Q_UNUSED(index)
@@ -1682,9 +1838,86 @@ bool DTabBar::canInsertFromMimeData(int index, const QMimeData *source) const
 
 void DTabBar::insertFromMimeData(int index, const QMimeData *source)
 {
-    d_func()->flashTabIndex = insertTab(index, source->text());
-    d_func()->makeVisible(d_func()->flashTabIndex);
-    d_func()->startTabFlash();
+    startTabFlash(insertTab(index, source->text()));
+}
+
+void DTabBar::insertFromMimeDataOnDragEnter(int index, const QMimeData *source)
+{
+    startTabFlash(insertTab(index, source->text()));
+}
+
+void DTabBar::tabInserted(int index)
+{
+    D_D(DTabBar);
+
+    d->tabMaximumSize.insert(index, QSize());
+    d->tabMinimumSize.insert(index, QSize());
+
+    d->QTabBar::tabInserted(index);
+
+    Q_EMIT tabIsInserted(index);
+}
+
+void DTabBar::tabLayoutChange()
+{
+    D_D(DTabBar);
+
+    d->QTabBar::tabLayoutChange();
+}
+
+void DTabBar::tabRemoved(int index)
+{
+    D_D(DTabBar);
+
+    d->tabMaximumSize.removeAt(index);
+    d->tabMinimumSize.removeAt(index);
+
+    d->QTabBar::tabRemoved(index);
+
+    Q_EMIT tabIsRemoved(index);
+}
+
+QSize DTabBar::tabSizeHint(int index) const
+{
+    D_DC(DTabBar);
+
+    QSize size = d->QTabBar::tabSizeHint(index);
+
+    QTabBarPrivate *dd = reinterpret_cast<QTabBarPrivate *>(qGetPtrHelper(d->d_ptr));
+    bool is_vertical = verticalTabs(dd->shape);
+
+    if (is_vertical) {
+        size.setWidth(qMax(size.width(), d->width()));
+    } else {
+        size.setHeight(qMax(size.height(), d->height()));
+    }
+
+    const QSize &min = qApp->buildDtkVersion() > DTK_VERSION_CHECK(2, 0, 8, 1) ? minimumTabSizeHint(index) : DTabBar::minimumTabSizeHint(index);
+    const QSize &max = qApp->buildDtkVersion() > DTK_VERSION_CHECK(2, 0, 8, 1) ? maximumTabSizeHint(index) : DTabBar::maximumTabSizeHint(index);
+
+    size.setWidth(qMax(size.width(), min.width()));
+    size.setHeight(qMax(size.height(), min.height()));
+
+    if (max.isValid()) {
+        size.setWidth(qMin(size.width(), max.width()));
+        size.setHeight(qMin(size.height(), max.height()));
+    }
+
+    return size;
+}
+
+QSize DTabBar::minimumTabSizeHint(int index) const
+{
+    D_DC(DTabBar);
+
+    return d->tabMinimumSize.value(index);
+}
+
+QSize DTabBar::maximumTabSizeHint(int index) const
+{
+    D_DC(DTabBar);
+
+    return d->tabMaximumSize.value(index);
 }
 
 DTabBarPrivate *DTabBar::d_func()
