@@ -235,7 +235,9 @@ bool DTitlebarPrivate::isVisableOnFullscreen()
 void DTitlebarPrivate::hideOnFullscreen()
 {
     D_Q(DTitlebar);
-    q->setProperty("_restore_height", q->height());
+    if (q->height() > 0) {
+        q->setProperty("_restore_height", q->height());
+    }
     q->setFixedHeight(0);
 }
 
@@ -265,8 +267,11 @@ void DTitlebarPrivate::updateFullscreen()
     } else {
         // must set to empty
         quitFullButton->show();
-        mainWindow->menuWidget()->setParent(nullptr);
-        mainWindow->setMenuWidget(Q_NULLPTR);
+        if (mainWindow->menuWidget()) {
+            mainWindow->menuWidget()->setParent(nullptr);
+            mainWindow->setMenuWidget(Q_NULLPTR);
+        }
+
         q->setParent(mainWindow);
         q->show();
         hideOnFullscreen();
@@ -353,12 +358,6 @@ void DTitlebarPrivate::handleParentWindowIdChange()
     D_Q(DTitlebar);
     if (!targetWindowHandle) {
         targetWindowHandle = targetWindow()->windowHandle();
-        targetWindowHandle->disconnect(targetWindow()->windowHandle(), &QWindow::windowStateChanged,
-                                       q, Q_NULLPTR);
-        targetWindowHandle->connect(targetWindow()->windowHandle(), &QWindow::windowStateChanged,
-        q, [ = ](Qt::WindowState) {
-            handleParentWindowStateChange();
-        });
     } else if (targetWindow()->windowHandle() != targetWindowHandle) {
         // Parent change???, show never here
         qWarning() << "targetWindowHandle change" << targetWindowHandle << targetWindow()->windowHandle();
@@ -668,6 +667,10 @@ bool DTitlebar::eventFilter(QObject *obj, QEvent *event)
             }
             break;
         }
+        case QEvent::WindowStateChange: {
+            d->handleParentWindowStateChange();
+            break;
+        }
         default:
             break;
         }
@@ -889,6 +892,8 @@ void DTitlebar::setVisible(bool visible)
         connect(this, SIGNAL(doubleClicked()), this, SLOT(_q_toggleWindowState()));
         connect(d->minButton, SIGNAL(clicked()), this, SLOT(_q_showMinimized()));
         connect(d->closeButton, &DWindowCloseButton::clicked, d->targetWindow(), &QWidget::close);
+
+        d->updateButtonsState(d->targetWindow()->windowFlags());
     } else {
         if (!d->targetWindow()) {
             return;
